@@ -111,7 +111,7 @@ void cut_to_eof(void)
  * copy_text is TRUE, copy the text back into the buffer afterward.
  * If cut_till_eof is TRUE, move all text from the current cursor
  * position to the end of the file into the cutbuffer. */
-void do_cut_text(bool copy_text, bool cut_till_eof)
+void do_cut_text(bool copy_text, bool marked, bool cut_till_eof)
 {
 #ifndef NANO_TINY
 	filestruct *cb_save = NULL;
@@ -125,12 +125,12 @@ void do_cut_text(bool copy_text, bool cut_till_eof)
 #endif
 	size_t was_totsize = openfile->totsize;
 
-	/* If a chain of cuts was broken, empty the cutbuffer. */
-	if (!keep_cutbuffer) {
+	/* If cuts were not continuous, or when cutting a region, clear the slate. */
+	if (!keep_cutbuffer || marked || cut_till_eof) {
 		free_filestruct(cutbuffer);
 		cutbuffer = NULL;
-		/* Indicate that future cuts should add to the cutbuffer. */
-		keep_cutbuffer = TRUE;
+		/* After a line cut, future line cuts should add to the cutbuffer. */
+		keep_cutbuffer = !marked && !cut_till_eof;
 	}
 
 #ifndef NANO_TINY
@@ -187,14 +187,6 @@ void do_cut_text(bool copy_text, bool cut_till_eof)
 		set_modified();
 
 	refresh_needed = TRUE;
-
-#ifdef ENABLE_COLOR
-	check_the_multis(openfile->current);
-#endif
-
-#ifdef DEBUG
-	dump_filestruct(cutbuffer);
-#endif
 }
 
 /* Move text from the current buffer into the cutbuffer. */
@@ -202,10 +194,10 @@ void do_cut_text_void(void)
 {
 #ifndef NANO_TINY
 	add_undo(CUT);
-#endif
-	do_cut_text(FALSE, FALSE);
-#ifndef NANO_TINY
+	do_cut_text(FALSE, openfile->mark, FALSE);
 	update_undo(CUT);
+#else
+	do_cut_text(FALSE, FALSE, FALSE);
 #endif
 }
 
@@ -215,7 +207,7 @@ void do_cut_text_void(void)
  * was moved, blow away previous contents of the cutbuffer. */
 void do_copy_text(void)
 {
-	static struct filestruct *next_contiguous_line = NULL;
+	static filestruct *next_contiguous_line = NULL;
 	bool mark_is_set = (openfile->mark != NULL);
 
 	/* Remember the current viewport and cursor position. */
@@ -227,7 +219,7 @@ void do_copy_text(void)
 	if (mark_is_set || openfile->current != next_contiguous_line)
 		cutbuffer_reset();
 
-	do_cut_text(TRUE, FALSE);
+	do_cut_text(TRUE, mark_is_set, FALSE);
 
 	/* If the mark was set, blow away the cutbuffer on the next copy. */
 	next_contiguous_line = (mark_is_set ? NULL : openfile->current);
@@ -245,7 +237,7 @@ void do_copy_text(void)
 void do_cut_till_eof(void)
 {
 	add_undo(CUT_TO_EOF);
-	do_cut_text(FALSE, TRUE);
+	do_cut_text(FALSE, FALSE, TRUE);
 	update_undo(CUT_TO_EOF);
 }
 #endif /* !NANO_TINY */
@@ -286,8 +278,4 @@ void do_uncut_text(void)
 
 	set_modified();
 	refresh_needed = TRUE;
-
-#ifdef ENABLE_COLOR
-	check_the_multis(openfile->current);
-#endif
 }
